<?php namespace RWMedia;

class Steps {
	
	public static function step1($data) {
		try {
			$databaseEntry = [
				'id' => $data->get('id'),
				'created' => REQUEST_TIME,
				'log' => json_encode($data->get('log')),
			];
			
			\RWMedia\Database::insert($databaseEntry, 'rwmedia');
			
			$data->log('Step 1: end, Resetting TTL counter and making a request for the next Steps');
			
			$data->set('ttl', $data->get('maxTtl'));
			
			self::prepareNewStepsRequest($data);
		} catch (Exception $e) {
			$data->log('Step 1: exception error: ' . $e->getMessage());	
			
			//try again if TTL left
			self::prepareNewStepsRequest($data, 4);
		}
	}

	public static function step2($data) {
		try {
			$url = substr($data->get('url'), 0, 2083); //cut URL after 2083 char, for compatibility and safety
			$data->set('url', $url);
			
			//test URL validity
			$responseArr = \RWHttp::request([
				'url' => $data->get('url'),
				'requestType' => 'HEAD',
			]);
			
			if ($responseArr['headers']['status-code'] === '200') {
				
				$databaseEntry = [
					'id' => $data->get('id'),
					'display_url' => $data->get('url'),
					'status' => 2,
					'log' => json_encode($data->get('log')),
				];
				
				\RWMedia\Database::insert($databaseEntry, 'rwmedia');
				
				$data->log('Step 2: end. Resetting TTL counter and making a request for the next Steps');	
				
				$data->set('ttl', $data->get('maxTtl'));
				self::prepareNewStepsRequest($data);
			}
			else {
				$data->log('Step 2: error, status-code not 200', [
					'headers' => $responseArr['headers'],
					'request' => $responseArr['request'],
					'response-raw-data' => $responseArr['data'],
				]);	
				
				//try again if TTL left
				self::prepareNewStepsRequest($data, 4);
			}
			
		} catch (Exception $e) {
			$data->log('Step 2: exception error: ' . $e->getMessage());	
			
			//try again if TTL left
			self::prepareNewStepsRequest($data, 4);
		} 
	}

	public static function step3($data, $entryObj) {
		try {
			$displayUrl = $entryObj->display_url;
			
			//analyze URL
			$analyzeOutput = \RWMedia\Analyze\url($displayUrl, $entryObj->id);
			$status = $analyzeOutput['status'];
			
			$data->log('Step 3: analyzed data: ', $analyzeOutput);
					
			$data->log("Step 3: end. Status set to ($status)");
			
			$databaseEntry = [
				'id' => $data->get('id'),
				'display_url' => $data->get('url'),
				'status' => $analyzeOutput['status'],
				'log' => json_encode($data->get('log')),
				
				'type' => $analyzeOutput['type'],
				'data_url' => $analyzeOutput['dataUrl'],
				'webm_url' => $analyzeOutput['webmUrl'],
				'mp4_url' => $analyzeOutput['mp4Url'],
				
				'title' => $analyzeOutput['title'],
				'video_duration' => $analyzeOutput['videoDuration'],
				'background_url' => $analyzeOutput['backgroundUrl'],
				'size' => $analyzeOutput['size'],
				'width' => $analyzeOutput['width'],
				'height' => $analyzeOutput['height'],
			];
			\RWMedia\Database::insert($databaseEntry, 'rwmedia');
			
			if ($status === 3) {
				$data->log("Step 3: Status=4 (processing/uploading), so resetting TTL counter and making a request for the next Steps");
				
				$data->set('ttl', $data->get('maxTtl') + 40); //TODO find a better way to regulate this for long uploads
				self::prepareNewStepsRequest($data, 5);
			}
			
		} catch (Exception $e) {
			$data->log('Step 3: exception error: ' . $e->getMessage());
			
			//try again if TTL left
			self::prepareNewStepsRequest($data, 4);
		} 
	}

	public static function step4($data, $entryObj) {
		
		try {
			$type = $entryObj->type;
			$uploadId = $entryObj->data_url;
			
			switch($type) {
				case 'video-streamable':
					$statusArr = \RWMedia\Analyze\streamableStatus($uploadId);
					
					// status 0 = Uploading
					// status 1 = Processing
					// status 2 = Ready
					// status 3 = Unavailable
					// status 4 = Not HTML 200
					$status = $statusArr['status'];
					
					$data->log("Streamable id=$uploadId");
					
					if ($status === 0 || $status === 1) {
						//request check again
						$percent = $statusArr['percent'];
						
						$data->log("Streamable status=$status, logging percentage ($percent)");
						
						$databaseEntry = [
							'id' => $data->get('id'),
							'upload_percentage' => $statusArr['percent'],
							'log' => json_encode($data->get('log')),
						];
						\RWMedia\Database::insert($databaseEntry, 'rwmedia');
						
						self::prepareNewStepsRequest($data, 5);
						return;
					}
					else if ($status === 3 || $status === 4) {
						//set status 9=failed, end task
						$data->log("Streamable status=$status, setting entry status to 9 (as failed) and ending the process");
						
						$databaseEntry = [
							'id' => $data->get('id'),
							'status' => 9,
							'upload_percentage' => $statusArr['percent'],
							'log' => json_encode($data->get('log')),
						];
						\RWMedia\Database::insert($databaseEntry, 'rwmedia');
					}
					else {
						//set status 8=finished, end task
						$data->log("Streamable status=$status, setting entry status to 8 (as successful) and ending the process");
						
						$databaseEntry = [
							'id' => $data->get('id'),
							'status' => 8,
							'upload_percentage' => $statusArr['percent'],
							
							'size' => $statusArr['size'],
							'video_duration' => $statusArr['duration'],
							'background_url' => 'https:' . $statusArr['thumbnail_url'],
							'log' => json_encode($data->get('log')),
						];
						\RWMedia\Database::insert($databaseEntry, 'rwmedia');
					}
					
					break;
				default:
					//set status 9=failed, end task
					$data->log("Cannot find '$type' type to continue processing or uploading, setting entry status to 9 (as failed) and ending the process");
		
					$databaseEntry = [
						'id' => $data->get('id'),
						'status' => 9,
						'upload_percentage' => 0,
						'log' => json_encode($data->get('log')),
					];
					\RWMedia\Database::insert($databaseEntry, 'rwmedia');
					break;
			}
			
		} catch (Exception $e) {
			$data->log('Step 4: exception error: ' . $e->getMessage());
			
			//try again if TTL left
			self::prepareNewStepsRequest($data, 4);
		}
	}
	
	public static function prepareNewStepsRequest($data, $delaySec = 0) {
		//delaySec = seconds to wait before sending the request
		
		$ttl = $data->get('ttl');
		
		//proceed if TTL left
		if ($ttl > 0) {
			$data->log("TTL left ($ttl), delaying $delaySec seconds and proceesing with the request.");
			
			if ($data->get('waitForResponse') === 0) {
				$data->log('waitForResponse set to 0, proceeding with async (non-blocking) request.');
			}
			
			$request = [
				'url' => $data->get('apiUrl'),
				'data' => json_encode([
					'url' => $data->get('url'),
					'apiUrl' => $data->get('apiUrl'),
					'id' => $data->get('id'),
					'ttl' => $data->get('ttl'),
					'maxTtl' => $data->get('maxTtl'),
					'nesting' => $data->get('nesting'),
					'waitForResponse' => $data->get('waitForResponse'),
					'log' => $data->get('log'),
					'appId' => $data->get('appId'),
				]),
				'requestType' => 'POST',
			];
			
			if ($data->get('waitForResponse') === 0) {
				\RWHttp::requestAsync($request, $delaySec);
			}
			else {
				$responseArr = \RWHttp::request($request, $delaySec);
			
				if ($responseArr['headers']['status-code'] === '200') {
					$data->log('STEPS API response received with 200 OK');
					
					$responseArr['data'] = json_decode($responseArr['data'], true);
					
					// combining elements from the received (and possibly nested) response Log array, with this Log array.
					$data->merge($responseArr['data']);
				}
				else {
					$data->log('STEPS API response received with error:', $responseArr);
					
					//try again if TTL left
					self::prepareNewStepsRequest($data);
				}
			}
			
		}
		else { // No TTL left, set status as Failed (9)
			$data->log('No TTL left, setting status to 9 (as failed) and ending the process');
			
			$databaseEntry = [
				'id' => $data->get('id'),
				'status' => 9,
				'log' => json_encode($data->get('log')),
			];
			
			\RWMedia\Database::insert($databaseEntry, 'rwmedia');
		}
	}

}

?>